module KR580VM80ALite
(
    input               clock,
    input               reset_n,
    input               ce,
    output              m0,
    output      [15:0]  address,
    input       [ 7:0]  in,
    output reg  [ 7:0]  out,
    output reg          we
);

assign address = sw ? cp : pc;
assign m0 = (t == 0);

// ----------------------------------------------
reg [15:0]  pc;
reg [15:0]  bc  = 16'hDEFF,
            de  = 16'hBEEF,
            hl  = 16'h0000,
            sp  = 16'h00FF;
reg [ 7:0]  a   = 8'hAF,
            psw = 8'b00000010;
// ----------------------------------------------
reg         sw;     // =1 Адрес указывает на CP, иначе =0 PC
reg [15:0]  cp;     // Адрес для считывания данных из памяти
reg [ 7:0]  opcode; // Сохраненный опкод
reg [ 4:0]  t;      // Исполняемый такт опкода [0..31]
// ----------------------------------------------
reg         b;      // =1 Запись d в 8-битный регистр n
reg         w;      // =1 Запись d в 16-битный регистр n
reg [15:0]  d;      // Данные
reg [ 2:0]  n;      // Номер регистра для записи
// ----------------------------------------------
wire [15:0] pcn = pc + 1;
// ----------------------------------------------

always @(posedge clock)
if (reset_n == 0) begin
    t  <= 0;        // Установить чтение кода на начало
    sw <= 0;        // Позиционировать память к PC
    pc <= 16'hF800; // Указатель на программу "Монитор"
end
else if (ce) begin

    t  <= t + 1;      // Счетчик микрооперации
    b  <= 0;          // Выключить запись в регистр (по умолчанию) 8bit
    w  <= 0;          // Выключить запись в регистр (по умолчанию) 16bit
    we <= 0;          // Аналогично, выключить запись в память (по умолчанию)

    // Запись опкода на первом такте выполнения инструкции
    if (m0) begin opcode <= in; pc <= pcn; end

    // Исполнение инструкции
    casex (t ? opcode : in)
    // 10T LXI R,**
    8'b00xx_0001: case (t)

        1: begin pc <= pcn; d[ 7:0] <= in; n <= opcode[5:4]; end
        2: begin pc <= pcn; d[15:8] <= in; w <= 1; end
        9: begin t <= 0; end

    endcase
    // 7T STAX B|D
    8'b000x_0010: case (t)

        0: begin we <= 1; out <= a; cp <= in[4] ? de : bc; sw <= 1; end
        6: begin sw <= 0; t <= 0; end

    endcase
    // 7T LDAX B|D
    8'b000x_1010: case (t)

        0: begin cp <= in[4] ? de : bc; sw <= 1; end
        1: begin b  <= 1; n <= 7; d <= in; end
        6: begin sw <= 0; t <= 0; end

    endcase
    // 5T DCX|INX R
    8'b00xx_x011: case (t)

        0: begin

            case (in[5:4])
            2'b00: begin d <= in[3] ? bc - 1 : bc + 1; end
            2'b01: begin d <= in[3] ? de - 1 : de + 1; end
            2'b10: begin d <= in[3] ? hl - 1 : hl + 1; end
            2'b11: begin d <= in[3] ? sp - 1 : sp + 1; end
            endcase

            n <= in[5:4];
            w <= 1;

        end
        4: begin t <= 0; end

    endcase

    endcase
end

// Запись данных в регистры
always @(posedge clock)
if (reset_n && ce) begin

    // 8-bit
    if (b)
    case (n)
    0: bc[15:8] <= d[7:0];  // B
    1: bc[ 7:0] <= d[7:0];  // C
    2: de[15:8] <= d[7:0];  // D
    3: de[ 7:0] <= d[7:0];  // E
    4: hl[15:8] <= d[7:0];  // H
    5: hl[ 7:0] <= d[7:0];  // L
    7:  a       <= d[7:0];  // A
    endcase

    // 16-bit
    if (w)
    case (n)
    2'b00: bc <= d;
    2'b01: de <= d;
    2'b10: hl <= d;
    2'b11: sp <= d;
    endcase

end

endmodule
